﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.Globalization;
using System.Windows.Forms;

namespace Bvews.Common
{
    static class LanguageSettingManager
    {
        private static readonly string path = Path.Combine(AppDataPath.CurrentVersion, "Language.txt");
        private static string language = string.Empty;
        private static bool languageChanged = false;

        public static bool Relaunch
        {
            get { return LanguageSettingManager.languageChanged; }
            set { LanguageSettingManager.languageChanged = value; }
        }

        public static bool IsJapanese
        {
            get { return Thread.CurrentThread.CurrentUICulture.Name.StartsWith("ja"); }
        }

        public static string CurrentLanguage
        {
            get { return Thread.CurrentThread.CurrentUICulture.Name; }
        }

        public static void Load()
        {
            try
            {
                language = File.ReadAllText(path);
            }
            catch
            {
                language = Thread.CurrentThread.CurrentUICulture.Name;
            }

            if (language.StartsWith("ja"))
            {
                Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja");
            }
            else
            {
                Thread.CurrentThread.CurrentUICulture = new CultureInfo("en");
            }
        }

        public static void Load(string language)
        {
            if (string.IsNullOrEmpty(language))
            {
                if (Thread.CurrentThread.CurrentUICulture.Name.StartsWith("ja"))
                {
                    Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja");
                }
                else
                {
                    Thread.CurrentThread.CurrentUICulture = new CultureInfo("en");
                }
            }
            else
            {
                if (language.StartsWith("ja"))
                {
                    Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja");
                }
                else
                {
                    Thread.CurrentThread.CurrentUICulture = new CultureInfo("en");
                }
            }
        }

        public static void Save(string language)
        {
            try
            {
                //File.WriteAllText(path, language);
            }
            catch
            {
            }

            if (language.StartsWith("ja"))
            {
                Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja");
            }
            else
            {
                Thread.CurrentThread.CurrentUICulture = new CultureInfo("en");
            }
        }

        public static void SetUpLanguageSettingToolStripMenuItems(Form owner, ToolStripMenuItem ja, ToolStripMenuItem en)
        {
            if (LanguageSettingManager.IsJapanese)
            {
                ja.Checked = true;
                en.Checked = false;
            }

            ja.Click += (s, e) =>
            {
                if (!ja.Checked)
                {
                    LanguageSettingManager.Save("ja");
                    if (MessageBox.Show(Resources.MessegeReLaunchApp, Resources.TextConfirm, MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        LanguageSettingManager.Relaunch = true;
                        owner.Close();
                    }
                }
                ja.Checked = true;
                en.Checked = false;
            };
            
            en.Click += (s, e) =>
            {
                if (!en.Checked)
                {
                    LanguageSettingManager.Save("en");
                    if (MessageBox.Show(Resources.MessegeReLaunchApp, Resources.TextConfirm, MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        LanguageSettingManager.Relaunch = true;
                        owner.Close();
                    }
                }
                ja.Checked = false;
                en.Checked = true;
            };
        }
    }
}
