﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

namespace Bvews.Common.UpdateChecker
{
    class UpdateInfo
    {
        public UpdateInfo()
        {
        }

        public event EventHandler OnUpdateInfoLoad = delegate { };

        public string LatestAppVersion { get; private set; }
        public bool IsLatest { get; private set; }
        public Dictionary<string, string> LatestChangelogMap { get; private set; }

        public void LoadUpdateInfoAsync(string versionInfoUrl)
        {
            LatestChangelogMap = new Dictionary<string, string>();
            IsLatest = true;

            try
            {
                // WebClient error when downloading file from https URL
                // https://stackoverflow.com/questions/39307684/webclient-error-when-downloading-file-from-https-url
                // System.Net.SecurityProtocolType.Tls12 definition not found
                // https://stackoverflow.com/questions/47269609/system-net-securityprotocoltype-tls12-definition-not-found
                ServicePointManager.SecurityProtocol = (SecurityProtocolType)(0xc0 | 0x300 | 0xc00);
                var wc = new WebClient();
                wc.DownloadDataCompleted += (s, e) =>
                {
                    if (e.Error != null)
                    {
                        return;
                    }

                    using (var ms = new MemoryStream(e.Result))
                    {
                        var data = UpdateInfoData.Load(ms);

                        LatestAppVersion = data.LatestAppVersion;
                        IsLatest = LatestAppVersion.CompareTo(System.Windows.Forms.Application.ProductVersion) <= 0;

                        LatestChangelogMap["ja"] = string.Empty;
                        LatestChangelogMap["en"] = string.Empty;

                        foreach (var line in data.ChangelogJapanese)
                        {
                            LatestChangelogMap["ja"] += line + "\r\n";
                        }

                        foreach (var line in data.ChangelogEnglish)
                        {
                            LatestChangelogMap["en"] += line + "\r\n";
                        }
                    }

                    OnUpdateInfoLoad(this, EventArgs.Empty);

                    wc.Dispose();
                };
                wc.DownloadDataAsync(new Uri(versionInfoUrl));
            }
            catch
            {
            }
        }
    }
}
