﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Bvews.Common.UpdateChecker
{
    public partial class UpdateNotificationForm : Form
    {
        public UpdateNotificationForm()
        {
            InitializeComponent();
        }

        public event EventHandler OnUpdateInfoLoad = delegate { };

        public bool DoNotShowAgain
        {
            get { return checkBoxDoNotShowAgain.Checked; }
            set { checkBoxDoNotShowAgain.Checked = value; }
        }
        public string VersionInfoUrl { get; private set; }
        public string AppDownloadUrl { get; private set; }
        public bool IsLatest { get; private set; }

        public void LoadUpdateInfoAsync(string appDownloadUrl)
        {
            var baseUri = new Uri(appDownloadUrl);
            var versionInfoUri = new Uri(baseUri, "version.xml");

            LoadUpdateInfoAsync(versionInfoUri.ToString(), appDownloadUrl);
        }

        public void LoadUpdateInfoAsync(string versionInfoUrl, string appDownloadUrl)
        {
            VersionInfoUrl = versionInfoUrl;
            AppDownloadUrl = appDownloadUrl;

            var updateInfo = new UpdateInfo();
            updateInfo.OnUpdateInfoLoad += (s, e) =>
            {
                IsLatest = updateInfo.IsLatest;
                var lang = Thread.CurrentThread.CurrentUICulture.Name;
                try { textBoxChangelog.Text = updateInfo.LatestChangelogMap[lang]; }
                catch { }
                textBoxChangelog.Select(0, 0);
                labelMessage.Text = string.Format(Bvews.Common.Resources.TextNewerVersionIsAvailable,
                    updateInfo.LatestAppVersion, Application.ProductVersion);

                OnUpdateInfoLoad(this, EventArgs.Empty);
            };
            updateInfo.LoadUpdateInfoAsync(versionInfoUrl);
        }

        private void buttonUpdateNow_Click(object sender, EventArgs e)
        {
            try { System.Diagnostics.Process.Start(AppDownloadUrl); }
            catch { }
            DialogResult = DialogResult.OK;
            Close();
        }

        private void buttonLater_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
