﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using System.Threading;
using System.Globalization;
using Bvews.Common;

namespace AudioLoopMaker
{
    public sealed class Settings
    {
        Preferences preferences = new Preferences();
        WaveViewOptions waveViewOptions = new WaveViewOptions();

        private static Settings instance = new Settings();

        private Settings()
        {
        }

        public static Settings Instance
        {
            get
            {
                return instance;
            }
        }

        public bool Upgraded { get; set; }

        public string Language { get; set; }

        public Preferences Preferences
        {
            get { return preferences; }
            set { preferences = value ?? new Preferences(); }
        }

        public WaveViewOptions WaveViewOptions
        {
            get { return waveViewOptions; }
            set { waveViewOptions = value ?? new WaveViewOptions(); }
        }

        public static void Load()
        {
            try
            {
                var serializer = new XmlSerializer(typeof(Settings));
                var path = Path.Combine(AppDataPath.CurrentVersion, "Settings.xml");
                using (var sr = new StreamReader(path))
                {
                    instance = (Settings)serializer.Deserialize(sr);
                }
            }
            catch (Exception)
            {
                instance = new Settings();
            }
        }

        public static void Save()
        {
            try
            {
                var serializer = new XmlSerializer(typeof(Settings));
                var path = Path.Combine(AppDataPath.CurrentVersion, "Settings.xml");
                using (var sw = new StreamWriter(path))
                {
                    serializer.Serialize(sw, instance);
                }
            }
            catch
            {
            }
        }

        public static void Upgrade()
        {
            if (!Instance.Upgraded)
            {
                if (AppDataPath.CurrentVersion != AppDataPath.PreviousVersion)
                {
                    var srcPaths = Directory.GetFiles(AppDataPath.PreviousVersion);
                    var dstPaths = srcPaths.Select(p => Path.Combine(AppDataPath.CurrentVersion, Path.GetFileName(p))).ToArray();
                    for (int i = 0; i < srcPaths.Length; i++)
                    {
                        File.Move(srcPaths[i], dstPaths[i]);
                    }
                    Instance.Upgraded = true;

                    Load();
                }
            }
        }
    }
}
