﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bvews.Wave
{
    public static class AudioBufferUtils
    {
        public static float[][] CreateAlternateLoop(float[][] buffers)
        {
            var channels = buffers.Length;
            var bufferLength = buffers.Min(b => b.Length);
            var resultLength = bufferLength * 2;

            var result = new float[channels][];
            for (int i = 0; i < channels; i++)
            {
                result[i] = new float[resultLength];
                buffers[i].CopyTo(result[i], 0);
                buffers[i].CopyTo(result[i], bufferLength);
                Array.Reverse(result[i], bufferLength, bufferLength);
            }

            return result;
        }

        public static void Amplify(ref float[][] buffers, float factor)
        {
            if (factor == 1)
            {
                return;
            }

            for (int i = 0; i < buffers.Length; i++)
            {
                for (int j = 0; j < buffers[i].Length; j++)
                {
                    buffers[i][j] = buffers[i][j] * factor;
                }
            }
        }

        public static double GetMaxAmplitude(float[][] buffers)
        {
            var result = 0.0;


            for (int i = 0; i < buffers.Length; i++)
            {
                for (int j = 0; j < buffers[i].Length; j++)
                {
                    result = Math.Max(result, Math.Abs(buffers[i][j]));
                }
            }

            return result;
        }

        public static double GetAverageAmplitude(float[][] buffers)
        {
            var result = 0.0;

            for (int i = 0; i < buffers.Length; i++)
            {
                for (int j = 0; j < buffers[i].Length; j++)
                {
                    result += Math.Abs(buffers[i][j]);
                }
            }

            var count = buffers.Sum(buffer => buffer.Length);

            if (count > 0)
            {
                return result / count;
            }
            else
            {
                return 0.0;
            }
        }

        public static float[] GenerateSineWave(double frequency, int sampleRate, double amplitude, double second)
        {
            float[] result = new float[(int)(second * sampleRate)];

            if (sampleRate > 0)
            {
                for (int i = 0; i < result.Length; i++)
                {
                    result[i] = (float)(amplitude * Math.Sin(frequency / (double)sampleRate * 2 * Math.PI * i));
                }
            }

            return result;
        }
        
        public static float[] FlattenToMono(float[][] buffers)
        {
            var channels = buffers.Length;
            var length = buffers.Min(b => b.Length);
            var result = new float[length];

            for (int i = 0; i < channels; i++)
            {
                for (int j = 0; j < length; j++)
                {
                    result[j] += buffers[i][j];
                }
            }

            for (int i = 0; i < length; i++)
            {
                result[i] /= channels;
            }

            return result;
        }
    }
}
