﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Bvews.Wave
{
    public class WaveFormat
    {
        short channels;
        int sampleRate;
        int averageBytesPerSecond;
        short blockAlign;
        short bitsPerSample;

        public WaveFormat()
        {
            FormatTag = WaveFormatTag.Pcm;
            Channels = 1;
            SampleRate = 48000;
            BitsPerSample = 16;
        }
        
        public WaveFormatTag FormatTag { get; set; }

        public short Channels
        {
            get
            {
                return channels;
            }
            set
            {
                channels = value;
                blockAlign = (short)(bitsPerSample / 8 * channels);
                averageBytesPerSecond = sampleRate * blockAlign;
            }
        }
        public int SampleRate
        {
            get
            {
                return sampleRate;
            }
            set
            {
                sampleRate = value;
                averageBytesPerSecond = sampleRate * blockAlign;
            }
        }
        public int AverageBytesPerSecond
        {
            get
            {
                return averageBytesPerSecond;
            }
            set
            {
                averageBytesPerSecond = value;
            }
        }
        public short BlockAlign
        {
            get
            {
                return blockAlign;
            }
            set
            {
                blockAlign = value;
            }
        }
        public short BitsPerSample
        {
            get
            {
                return bitsPerSample;
            }
            set
            {
                bitsPerSample = value;
                blockAlign = (short)(bitsPerSample / 8 * channels);
                averageBytesPerSecond = sampleRate * blockAlign;
            }
        }

        public byte[] GenerateBytes()
        {
            var result = new List<byte>();

            result.AddRange(BitConverter.GetBytes((short)FormatTag));
            result.AddRange(BitConverter.GetBytes((ushort)Channels));
            result.AddRange(BitConverter.GetBytes((uint)SampleRate));
            result.AddRange(BitConverter.GetBytes((uint)AverageBytesPerSecond));
            result.AddRange(BitConverter.GetBytes((ushort)BlockAlign));
            result.AddRange(BitConverter.GetBytes((ushort)BitsPerSample));

            return result.ToArray();
        }
    }
}

