﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Bvews.Wave
{
    public class WaveInfo
    {
        public WaveInfo()
        {
            ChunkRiff = new RiffChunk();
            ChunkRiff.SetId("RIFF");
            ChunkRiff.SetFormType("WAVE");

            ChunkFmt = new RiffChunk();
            ChunkFmt.SetId("fmt ");

            ChunkData = new RiffChunk();
            ChunkData.SetId("data");

            WaveFormat = new Wave.WaveFormat();

            IsLoadSuccessful = true;
        }


        public WaveInfo(string path)
        {
            IsLoadSuccessful = false;
            Path = null;

            FileStream fs = null;
            try
            {
                ChunkRiff = new RiffChunk();
                ChunkFmt = null;
                ChunkData = null;

                fs = new FileStream(path, FileMode.Open, FileAccess.Read);
                using (BinaryReader br = new BinaryReader(fs))
                {
                    // ヘッダの読み込み
                    ChunkRiff = GetChunk(fs, br);
                    if (!ChunkRiff.IsIdEqual("RIFF") || !ChunkRiff.IsFormTypeEqual("WAVE"))
                    {
                        throw new ArgumentException();
                    }

                    fs.Position = fs.Position + 4;

                    // TODO: Consider "LIST" chunk existance.

                    // C# checking for binary reader end of file
                    // https://stackoverflow.com/questions/10942848/c-sharp-checking-for-binary-reader-end-of-file
                    while (br.BaseStream.Position < br.BaseStream.Length)
                    {
                        var chunk = GetChunk(fs, br);

                        if (chunk.IsIdEqual("fmt "))
                        {
                            ChunkFmt = chunk;

                            var format = new WaveFormat();
                            var formatTag = br.ReadUInt16();
                            if(formatTag == 0xFFFE)
                            {
                                format.FormatTag = WaveFormatTag.Pcm;
                            }
                            else
                            {
                                format.FormatTag = (WaveFormatTag)formatTag;
                            }
                            format.Channels = (short)br.ReadUInt16();
                            format.SampleRate = (int)br.ReadUInt32();
                            format.AverageBytesPerSecond = (int)br.ReadUInt32();
                            format.BlockAlign = (short)br.ReadUInt16();
                            format.BitsPerSample = (short)br.ReadUInt16();
                            WaveFormat = format;
                        }
                        else if (chunk.IsIdEqual("data"))
                        {
                            ChunkData = chunk;
                        }

                        fs.Seek(chunk.Position + chunk.Size, SeekOrigin.Begin);
                    }

                    IsLoadSuccessful = ChunkRiff != null && ChunkFmt != null && ChunkData != null;
                    Path = path;
                    return;
                }
            }
            catch
            {
                IsLoadSuccessful = false;
                Path = null;
                throw;
            }
            finally
            {
                if (fs != null)
                {
                    fs.Dispose();
                }
            }
        }

        public RiffChunk ChunkRiff { get; private set; }
        public RiffChunk ChunkFmt { get; private set; }
        public RiffChunk ChunkData { get; private set; }
        public WaveFormat WaveFormat { get; private set; }
        public bool IsLoadSuccessful { get; private set; }
        public string Path { get; private set; }

        public void SetWaveFormat(WaveFormat waveFormat)
        {
            WaveFormat = waveFormat;
        }

        private static RiffChunk GetChunk(FileStream fs, BinaryReader br)
        {
            var chunk = new RiffChunk();

            chunk.Id = br.ReadBytes(4);
            chunk.Size = br.ReadUInt32();
            chunk.Position = fs.Position;
            if (chunk.Size >= 4)
            {
                chunk.FormType = br.ReadBytes(4);
                fs.Position = fs.Position - 4;
            }

            return chunk;
        }
    }
}
