﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AudioLoopMaker
{
    abstract class WaveStream
    {
        protected int start;
        protected int end;

        protected float[][] buffers;
        protected int channels;
        protected int bufferLength;
        protected int bufferPosition;

        public WaveStream(float[][] buffers)
        {
            Buffers = buffers;
            Start = 0;
            End = bufferLength;
        }

        public int Start
        {
            get { return start; }
            set { start = value > 0 ? value : 0; }
        }

        public int End
        {
            get { return end; }
            set { end = value < bufferLength ? value : bufferLength; }
        }

        public int BufferPosition
        {
            get { return bufferPosition; }
            set { bufferPosition = value; }
        }

        public float[][] Buffers
        {
            get { return buffers; }
            set
            {
                if(value == null)
                {
                    throw new ArgumentNullException();
                }
                buffers = value;
                channels = buffers.Length;
                bufferLength = buffers.Min(b => b.Length);
            }
        }

        public float[][] Read(int length)
        {
            var result = new float[channels][];

            if (length <= 0)
            {
                for (int i = 0; i < channels; i++)
                {
                    result[i] = new float[0];
                }
                return result;
            }

            for (int i = 0; i < channels; i++)
            {
                result[i] = new float[length];
            }

            if (bufferLength <= 0 || end - start <= 0)
            {
                return result;
            }

            return Read(result, length);
        }

        abstract protected float[][] Read(float[][] buffers, int length);

        abstract public float[][] Flush();
    }
}
