﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AudioLoopMaker
{
    class WaveStreamAlternateLoop : WaveStream
    {
        bool isReverse;

        public WaveStreamAlternateLoop(float[][] buffers)
            : base(buffers) { }

        protected override float[][] Read(float[][] resultBuffers, int length)
        {
            for (int i = 0; i < length; i++)
            {
                if (bufferPosition >= end)
                {
                    isReverse = true;
                    bufferPosition = end <= 0 ? 0 : end - 1;
                }
                else if (bufferPosition < start)
                {
                    isReverse = false;
                    bufferPosition = start;
                }

                for (int j = 0; j < channels; j++)
                {
                    resultBuffers[j][i] = buffers[j][bufferPosition];
                }

                if (isReverse)
                {
                    bufferPosition--;
                }
                else
                {
                    bufferPosition++;
                }
            }

            return resultBuffers;
        }

        public override float[][] Flush()
        {
            var length = end - start;
            var result = new float[channels][];

            for (int i = 0; i < channels; i++)
            {
                result[i] = new float[length * 2];
                Array.Copy(buffers[i], start, result[i], 0, length);
                Array.Copy(buffers[i], start, result[i], length, length);
                Array.Reverse(result[i], length, length);
            }

            return result;
        }
    }
}
